###############################################################################
##  Kroller - v0.91
###############################################################################
##  
##  Code by:
##    Brandon Zehm <caspian at dotconf.net>
##  
##  Icon set arranged by:
##    Adam Duskett
##  
##  Kroller is an extension for SuperKaramba to make a MacOS-X like
##  menu bar.
##  It was originally based on tuxbar, but now only a few lines of
##  code from it remain.
##  
##  This SuperKaramba extension was designed to be flexible.
##  You can add new icons, or change existing ones by simply
##  editing the lines in the "User Configurable Section" in this
##  file.  You can use icons of any size you want.  All you need
##  to do is make sure that all the "small" icons are the same
##  size, and all the "big" icons are the same size, and you set
##  those sizes in the imageSize and imageSizeBig variables.
##
##  Feel free to email me with comments or suggestions!
##
###############################################################################

## Import a few standard python modules
import string, re, os, time


## This import statement allows access to the karamba functions
import karamba





############################
##  Global Configuration  ##
############################
conf = {
    
    ## Configuration file location, DO NOT NORMALLY EDIT THIS!  Recent enhancements
    ## in SuperKaramba allow us to locate the conf file on our own.  If you
    ## do hard-code a path here it will still work, but you shouldn't need to.
    "configurationFile"      : "kroller.conf",
    
    "version"                : "v0.91",
    
    ## The following variables are overwritten by the configuration file
    "animateOnClick"         : 0,                         ## Animate icons when clicked
    "detectResolution"       : 0,                         ## Auto detection of resolution
    "resolutionX"            : 0,                         ## Manually specified X resolution
    "resolutionY"            : 0,                         ## Manually specified Y resolution
    "autoPosition"           : 0,                         ## Automatically position kroller or not
    "maxFPS"                 : 0,                         ## Maximum widget updates / second.  0 == no limit.
    "smallImageSize"         : 0,                         ## Size of small images
    "largeImageSize"         : 0,                         ## Size of large images
    "iconSeparation"         : "default",                 ## Icon Separation
    "currentIconEffect"      : "none",                    ## Effect for current icon
    "useBackgroundBar"       : 0,                         ## Enable/disable background bar image
    "barImages"              : "barimages/",              ## Directory where to find bar images
    "backgroundBarColor"     : "",                        ## Color or background bar.. this need to be a directory under the barImages dir!
    "autoHide"               : 0,                         ## Enable/disable auto-hiding
    "autoHideDelay"          : 0,                         ## Seconds of idle time to activate autoHide
    "virtualDesktopSelector" : 0,                         ## Enable/disable virtual desktop selector
    "numVirtualDesktops"     : 0,                         ## Number of virtual desktops to display
    "virtualDesktopIconX"    : 0,                        ## Pixels wide the virtual desktop image is
    "virtualDesktopIconY"    : 0,                        ## Pixels tall the virtual desktop image is
    "debug"                  : 0,                         ## Debug level
    
}


## Define some more global variables
## The values are not set here!

## Icon related vars
commands         = []
imageFiles       = []
imageFilesBig    = []
images           = []
imagesBig        = []
imageSize        = 0
imageSizeBig     = 0
numIcons         = 0

## Icon placement variables
iconSeparation   = 0
centerSmallIcon  = 0
iconFloor        = 0
raiseIcons       = 0
xZoomBoundaryMin = 0
xZoomBoundaryMax = 0
yZoomBoundaryMin = 0
yZoomBoundaryMax = 0
    

## Widget parameters
borderLeft       = 0
borderRight      = 0
borderBottom     = 0
widgetWidth      = 0
widgetHeight     = 0
resolutionX      = 0
resolutionY      = 0

## Icon resizing related parameters
sizeDiff         = 0
sizeRatio        = 0

## Time related vars
sleepTime        = 0
lastActiveTime   = 0

## Variables for background bar support
useBackBar       = 0
backBarLeft      = 0
backBarCenter    = 0
backBarRight     = 0
backBarBorder    = 4

## These are used for storing current icon placement info
currentXPosition = 0
currentYPosition = 0
currentSize      = 0

## Virtual Desktop Related
virtualDesktop   = []
xVdBoundaryMin   = 0
xVdBoundaryMax   = 0
yVdBoundaryMin   = 0
yVdBoundaryMax   = 0












###############################################################################
##  Function:
##    initWidget(widget)
##  
##  Description:
##    This function is buggy.. I think it gets called too early, because
##    if you try to create an image it crashes karamba.
###############################################################################
def initWidget(widget):
    """Initialize Widget"""    
    
    global conf
    if (conf['debug'] >= 3):
        print "initWidget() got called"
    
    global images
    global imagesBig
    global commands
    global imageFiles
    global imageFilesBig
    global images
    global imagesBig
    global imageSize
    global imageSizeBig
    global numIcons
    global iconSeparation
    global centerSmallIcon
    global borderLeft
    global borderRight
    global borderBottom
    global sizeDiff
    global sizeRatio
    global iconFloor
    global raiseIcons
    global xZoomBoundaryMin
    global xZoomBoundaryMax
    global yZoomBoundaryMin
    global yZoomBoundaryMax
    global widgetHeight
    global widgetWidth
    global sleepTime
    global lastActiveTime
    global useBackBar
    global backBarLeft
    global backBarCenter
    global backBarRight
    global resolutionX
    global resolutionY
    global virtualDesktop
    global xVdBoundaryMin
    global xVdBoundaryMax
    global yVdBoundaryMin
    global yVdBoundaryMax
    
    ###################################
    ##  Read the configuration file  ##
    ###################################
    ## Try hard to find the configuration file, otherwise exit with an error message
    themePath = karamba.getThemePath(widget)
    if (os.path.exists(os.getenv("HOME") + "/.superkaramba/" + conf["configurationFile"])):
        print "Loading configuration from home directory: ", os.getenv("HOME") + "/.superkaramba/" + conf["configurationFile"]
        readConfigurationFile(os.getenv("HOME") + "/.superkaramba/" + conf["configurationFile"])
    elif (os.path.exists(karamba.getThemePath(widget) + "/" + conf["configurationFile"])):
        print "Loading configuration from theme directory: ", karamba.getThemePath(widget) + "/" + conf["configurationFile"]
        readConfigurationFile(karamba.getThemePath(widget) + "/" + conf["configurationFile"])
    elif (os.path.exists(conf["configurationFile"])):
        print "Loading configuration from manually specified location: ", conf["configurationFile"]
        readConfigurationFile(conf["configurationFile"])
    else:
        print ""
        print "ERROR - Loading the configuration file failed!  The configuration file could not be located!"
        print "Hint, try putting kroller.conf in ~/.superkaramba/"
        print ""
        sys.exit(1)
    
    
    ## Make sure numeric configuration parameters are set that way
    imageSize    = int(conf['smallImageSize'])
    imageSizeBig = int(conf['largeImageSize'])
    conf['debug'] = int(conf['debug'])
    conf['autoHide'] = int(conf['autoHide'])
    conf['autoHideDelay'] = int(conf['autoHideDelay'])
    conf['animateOnClick'] = int(conf['animateOnClick'])
    conf["virtualDesktopSelector"] = int(conf["virtualDesktopSelector"])
    conf["numVirtualDesktops"] = int(conf["numVirtualDesktops"])
    conf["virtualDesktopIconX"] = int(conf["virtualDesktopIconX"])
    conf["virtualDesktopIconY"] = int(conf["virtualDesktopIconY"])
    useBackBar = int(conf['useBackgroundBar'])
    
    ## <calculations>
    
    ## Calculate some other important stuff
    if (re.search("default", str(conf['iconSeparation']))):
        iconSeparation = (imageSize + imageSizeBig)/2            ## iconSeparation is the size of the "cells" where icons go
        if (iconSeparation > (2*imageSize)):
            iconSeparation = (2*imageSize)
    else:
        if (re.search("\D", str(conf['iconSeparation']))):
            print "ERROR - The iconSeparation parameter in the configuration file is not \"default\" or a numeric value!"
        else:
            iconSeparation = int(conf['iconSeparation'])
            iconSeparation = iconSeparation + imageSize
    
    numIcons = len(imageFiles)                               ## Self explanitory
    centerSmallIcon = ((iconSeparation/2)-(imageSize/2))     ## The number of pixels to center each small icon in it's "cell"
    borderLeft = iconSeparation                              ## How much "white" space on the left
    borderRight = iconSeparation                             ## How much "white" space on the left
    if (conf["virtualDesktopSelector"] == 1):                ## If virtual desktop selector is enabled make room for it
        conf["virtualDesktopIconY"] = imageSize/2
        conf["virtualDesktopIconX"] = int(conf["virtualDesktopIconY"] * (4.0/3))  ## Use a 4/3 aspect ratio for virtual desktop icons
        borderRight += (conf["virtualDesktopIconX"] * (conf["numVirtualDesktops"]/2)) 
        if (useBackBar == 1):
            borderRight += imageSize
    borderBottom = 10                                        ## How much "white" space on the bottom
    sizeDiff = imageSizeBig - imageSize                      ## How many pixels are there from the small icon to the big icon
    sizeRatio = ((sizeDiff + 0.0) / imageSizeBig)            ## Ratio of small icon to big icon
    raiseIcons = 30                                          ## How far to "raise" icons up when icon is at its largest size
    
    ## Calculate widget size
    widgetWidth = (borderLeft + (iconSeparation * numIcons) + borderRight)
    widgetHeight = imageSizeBig + raiseIcons + borderBottom
    
    ## Figure out where the bottom of the icons should be (the iconFloor)
    iconFloor = (widgetHeight - imageSize - borderBottom)
    
    ## Boundaries in which we do icon zooming
    ## These will probably have to become more complex formulas in the future
    xZoomBoundaryMin = (borderLeft-iconSeparation)
    xZoomBoundaryMax = (borderLeft+(iconSeparation*(numIcons+1)))
    yZoomBoundaryMin = 0
    yZoomBoundaryMax = widgetHeight
    
    ## Sleep time based on maxFPS from configuration file
    if (conf['maxFPS'] != 0):
        sleepTime = (1.0 / conf['maxFPS'])
    
    ## If we're in debugging mode display the current kroller environment
    if (conf['debug'] >= 1):
        print "numIcons: ", numIcons
        print "imageSize: ", imageSize
        print "imageSizeBig:", imageSizeBig
        print "centerSmallIcon: ", centerSmallIcon
        print "iconSeparation: ", iconSeparation
        print "iconFloor: ", iconFloor
        print "sizeDiff: ", sizeDiff
        print "sizeRatio: ", sizeRatio
        print "borderLeft: ", borderLeft
        print "borderRight: ", borderRight
        print "borderBottom: ", borderBottom
        print "raiseIcons: ", raiseIcons
        print "widgetWidth: ", widgetWidth
        print "widgetHeight: ", widgetHeight
        print "sleepTime: ", sleepTime
        
    ## </calculations>
    
    ###################################
    ##  Detect current X resolution  ##
    ###################################
    if (int(conf['detectResolution']) == 1):
        detectResolution()
    else:
        resolutionX = int(conf['resolutionX'])
        resolutionY = int(conf['resolutionY'])
        print "Using specified resolution: " + str(resolutionX) + "x" + str(resolutionY)
    
    ## Set the size of our widget since we're just starting
    karamba.resizeWidget(widget, widgetWidth, widgetHeight)
    
    
    ################################
    ##  Setup the background bar  ##
    ################################
    if (useBackBar == 1):
        ## Create image references
        backBarLeft =   (karamba.createImage(widget, 0, 0, conf["barImages"] + conf["backgroundBarColor"] + "/background-left.png"))
        backBarCenter = (karamba.createImage(widget, 0, 0, conf["barImages"] + conf["backgroundBarColor"] + "/background-center.png"))
        backBarRight =  (karamba.createImage(widget, 0, 0, conf["barImages"] + conf["backgroundBarColor"] + "/background-right.png"))
        ## Set initial sizes
        karamba.resizeImageSmooth(widget, backBarLeft, imageSize + (backBarBorder * 2), imageSize + (backBarBorder * 2))
        karamba.resizeImage(widget, backBarCenter, widgetWidth - borderLeft - borderRight, imageSize + (backBarBorder * 2))
        karamba.resizeImageSmooth(widget, backBarRight, imageSize + (backBarBorder * 2), imageSize + (backBarBorder * 2))
        ## Set initial positions
        karamba.moveImage(widget, backBarLeft, borderLeft - imageSize - (backBarBorder * 2), iconFloor - backBarBorder)
        karamba.moveImage(widget, backBarCenter, borderLeft, iconFloor - backBarBorder)
        karamba.moveImage(widget, backBarRight, borderLeft + (numIcons * iconSeparation), iconFloor - backBarBorder)
    
    
    ######################################
    ##  Setup Virtual Desktop Selector  ##
    ######################################
    if (conf["virtualDesktopSelector"] == 1):
        
        ## Setup some variables
        curicon = "top"
        initialDesktopNumTop = 1
        initialDesktopNumBottom = (conf["numVirtualDesktops"]/2) + 1  ## FIXME this will only work if there are an even number of virtual desktops being displayed
        
        ## Setup the background bar for the virtual desktop area
        if (useBackBar == 1):
            backBarVDCenter =   (karamba.createImage(widget, 0, 0, conf["barImages"] + conf["backgroundBarColor"] + "/background-center.png"))
            backBarVDRight  =   (karamba.createImage(widget, 0, 0, conf["barImages"] + conf["backgroundBarColor"] + "/background-right.png"))
            karamba.resizeImage(widget, backBarVDCenter, (conf["virtualDesktopIconX"] * (conf["numVirtualDesktops"]/2)) + iconSeparation, imageSize + (backBarBorder * 2))
            karamba.resizeImageSmooth(widget, backBarVDRight, imageSize + (backBarBorder * 2), imageSize + (backBarBorder * 2))
            karamba.moveImage(widget, backBarVDCenter, borderLeft + (iconSeparation*(numIcons)), iconFloor - backBarBorder)
            karamba.moveImage(widget, backBarVDRight, borderLeft + (iconSeparation*(numIcons+1)) + (conf["virtualDesktopIconX"] * (conf["numVirtualDesktops"]/2)), iconFloor - backBarBorder)
        
        ## Create enough empty entries in virtualDesktop
        for i in range(0, conf["numVirtualDesktops"] + 1, 1):
            virtualDesktop.append(0)
            
        ## For each virtual desktop (VD) place an icon on the screen and assign a command to run when clicked
        for i in range(0, conf["numVirtualDesktops"], 1):
            x = borderLeft + (iconSeparation*(numIcons+1)) + (int(i/2)*conf["virtualDesktopIconX"])
            y = widgetHeight - borderBottom - (conf["virtualDesktopIconY"]*2)
            
            if (curicon == "top"):
                desktopNum = initialDesktopNumTop
                initialDesktopNumTop = initialDesktopNumTop + 1
                curicon = "bottom"
            else:
                y = y + conf["virtualDesktopIconY"]
                desktopNum = initialDesktopNumBottom
                initialDesktopNumBottom = initialDesktopNumBottom + 1
                curicon = "top"
            
            ## Debugging
            if (conf["debug"] >= 1):
                print "i", 1, " and ", x, "x", y, "desktopNum", desktopNum
            
            virtualDesktop[desktopNum] = (karamba.createImage(widget, 0, 0, conf["barImages"] + "virtual_desktop.png"))
            karamba.resizeImage(widget, virtualDesktop[desktopNum],   conf["virtualDesktopIconX"], conf["virtualDesktopIconY"])
            karamba.moveImage(widget, virtualDesktop[desktopNum], x, y)
            karamba.createClickArea(widget, x, y, conf["virtualDesktopIconX"], conf["virtualDesktopIconY"], "dcop kwin KWinInterface setCurrentDesktop " + str(desktopNum))
        
        ## These vars are used in widgetMouseMoved() but pre-calculated here once for CPU savings
        xVdBoundaryMin = borderLeft + (iconSeparation*(numIcons+1))
        xVdBoundaryMax = borderLeft + (iconSeparation*(numIcons+1)) + ((conf["numVirtualDesktops"]/2) * conf["virtualDesktopIconX"])
        yVdBoundaryMin = widgetHeight - borderBottom - (conf["virtualDesktopIconY"]*2) - conf["virtualDesktopIconY"]
        yVdBoundaryMax = widgetHeight - borderBottom
        
        
    
    ##########################################
    ##  Setup All the normal rolling icons  ##
    ##########################################
    for i in range (0,numIcons):
        ## Add small image
        images.append(karamba.createImage(widget, borderLeft + centerSmallIcon + (i*iconSeparation), iconFloor, imageFiles[i]))
        ## Add large image
        imagesBig.append(karamba.createImage(widget, borderLeft + (i*iconSeparation), widgetHeight, imageFilesBig[i]))
        ## Add clickable area that runs a command
        karamba.createClickArea(widget, borderLeft + (i*iconSeparation), (widgetHeight - borderBottom - imageSizeBig - raiseIcons), iconSeparation, imageSizeBig + (raiseIcons/2), commands[i])
        ## Resize the image just in case the image wasn't imageSize pixels wide
        karamba.resizeImage(widget, images[i], imageSize, imageSize)
    
    
    #######################################################
    ##  Place widget at the bottom center of the screen  ##
    #######################################################
    if (int(conf['autoPosition']) == 1):
        karamba.moveWidget(widget, ((resolutionX/2) - (widgetWidth/2)), (resolutionY-widgetHeight) )
    
    ## Update last active time if we're autoHiding (without this autoHiding never starts)
    if (conf['autoHide'] == 1):
        lastActiveTime = time.time()
    
    karamba.redrawWidget(widget)
    print "Kroller successfully initiated!"










###############################################################################
##  Function: 
##    widgetClicked(widget, x, y, button)
##  
##  This gets called everytime our widget is clicked.
##  Notes:
##    widget = reference to our widget
##    x = x position (relative to our widget)
##    y = y position (relative to our widget)
##    botton = button clicked:
##                    1 = Left Mouse Button
##                    2 = Middle Mouse Button
##                    3 = Right Mouse Button, but this will never happen
##                        because the right mouse button brings up the
##                        Karamba menu.
###############################################################################
def widgetClicked(widget, x, y, button):
    """widgetClicked gets called when our widget is clicked"""
    global currentXPosition
    global currentYPosition
    global currentSize
    global lastActiveTime
    
    if (conf['debug'] >= 3):
        print "widgetClicked() got called"
    
    ## Unhide if we need to
    if (lastActiveTime):
        if ((time.time() - lastActiveTime) > conf['autoHideDelay']):
            raiseWidget(widget)
        
        ## Update last active time
        lastActiveTime = time.time()
    
    
    ## LEFT CLICK
    ## Animate icon when an application is launched
    if (button == 1 and (conf['animateOnClick'] == 1)):
        iconNum = (x - borderLeft) / iconSeparation
        if ((iconNum >= 0) and (iconNum < numIcons)):
            ## Going down
            for down in range (1,raiseIcons,4):
                karamba.moveImage(widget, imagesBig[iconNum], currentXPosition, currentYPosition + down)
                karamba.redrawWidget(widget)
                ## Sleep a bit to accomadate the maxFPS setting
                if (sleepTime):
                    time.sleep(sleepTime)
            ## Going up
            for up in range (1,raiseIcons,4):
                karamba.moveImage(widget, imagesBig[iconNum], currentXPosition, currentYPosition + raiseIcons - up)
                karamba.redrawWidget(widget)
                ## Sleep a bit to accomadate the maxFPS setting
                if (sleepTime):
                    time.sleep(sleepTime)
        
        ## Update the virtual desktop selector
        if (conf["virtualDesktopSelector"] == 1):
            if (sleepTime):
                time.sleep(0.1)
            ## Get current virtual desktop
            fp = os.popen("dcop kwin KWinInterface currentDesktop")
            output = fp.read()
            output = output.splitlines()
            karamba.changeImageIntensity(widget, virtualDesktop[int(output[0])], 0.7, 200)
            karamba.redrawWidget(widget)
    
    
    ## MIDDLE CLICK
    ## Show off if middle clicked (this will be replaced with something else in the future!)
    if (button == 2):
        for i in range (x, (borderLeft + ((numIcons+1) * iconSeparation) - 1), 10):
            widgetMouseMoved(widget, i, (widgetHeight/2), 0)
        for i in range ((borderLeft + ((numIcons+1) * iconSeparation)), (borderLeft-iconSeparation), -10):
            widgetMouseMoved(widget, i, (widgetHeight/2), 0)
        for i in range ((borderLeft-iconSeparation), x, 10):
            widgetMouseMoved(widget, i, (widgetHeight/2), 0)
    
    








###############################################################################
##  Function:
##    widgetUpdated(widget)
##  
##  Description:
##    Right now this function is only used for initializing the widget
##
###############################################################################
def widgetUpdated(widget):
    """widgetUpdated gets called at program init and every 4 seconds or so"""    
    
    
    if (conf['debug'] >= 1):
        print "widgetUpdate got called"
    
    ## Check to see if we need to autoHide
    if (lastActiveTime):
        if ((time.time() - lastActiveTime) >= conf['autoHideDelay']):
            resetWidget(widget)
            hideWidget(widget)
    
    
        












###############################################################################
##  Function:   
##    widgetMouseMoved(widget, x, y, button)
##  
##  This gets called everytime the mouse is moved over our widget
##  Notes
##    widget = reference to our widget
##    x = x position (relative to our widget)
##    y = y position (relative to our widget)
##    botton = button being held:
##                    0 = No Mouse Button
##                    1 = Left Mouse Button
##                    2 = Middle Mouse Button
##                    3 = Right Mouse Button, but this will never happen
##                        because the right mouse button brings up the
##                        Karamba menu.
###############################################################################
def widgetMouseMoved(widget, x, y, button):
    """widgetMouseMoved gets called everytime the mouse gets moved over our widget"""    
    global currentXPosition
    global currentYPosition
    global currentSize
    global lastActiveTime
    
    ## Unhide if we need to
    if (lastActiveTime):
        if ((time.time() - lastActiveTime) >= conf['autoHideDelay']):
            raiseWidget(widget)
        
        ## Update last active time
        lastActiveTime = time.time()
    
    
    iconNum = -2
    
    ## If the mouse is in the target area
    if (x >= xZoomBoundaryMin) and (x < xZoomBoundaryMax) and (y >= yZoomBoundaryMin) and (y <= yZoomBoundaryMax):
        
        if (conf['debug'] >= 2):
            print "processing.."
        
        ## Which icon is the mouse over?
        iconNum = (x - borderLeft) / iconSeparation   ## borderLeft is initial white-space, and iconSeparation pixels between each icon
        
        ## Unless the current icon comes up as a non-valid number
        if (-2 < iconNum < numIcons+1):
            
            ## This returns a number between 0 and iconSeparation depending on how far through the image you are
            offset              = (x - borderLeft) % iconSeparation         ## 0 - iconSeparation
            offsetRatio         = (offset/(iconSeparation + 0.0) )          ## Ratio from 0-1 of the current offset
            offsetImageSizeBig  = (offsetRatio * (imageSizeBig/2))          ## 0 - (imageSizeBig/2)
            offsetSizeDiff      = (offsetRatio * sizeDiff)                  ## 0 - sizeDiff
            ## yRatio lets us do smooth scrolling from the top and bottom
            ## You will see it throughout the script for this reason
            ## If y is in the top half of the icon
            if (y < widgetHeight - borderBottom - raiseIcons - (imageSizeBig/2)):
                yRatio          = ((y + 0.0) / (widgetHeight - borderBottom - raiseIcons - (imageSizeBig/2)))
            else: 
                yRatio = 1
            
            ## If we're positioning the first or last icon, do a special formula
            if (iconNum == -1):
                xShiftLeft = (offset - iconSeparation) * yRatio
                xShiftRight = 0
            elif (iconNum == numIcons):
                xShiftRight = offset * yRatio
                xShiftLeft = 0
            else:
                xShiftLeft = 0
                xShiftRight = 0
                
            
            ## Resize the background bar if we're using it
            if (useBackBar == 1):
                karamba.moveImage(widget, backBarLeft, borderLeft - imageSize - (backBarBorder * 2) - ((iconSeparation/2) * yRatio) - (xShiftLeft/2) + (xShiftRight/2), iconFloor - backBarBorder)
                karamba.moveImage(widget, backBarCenter, borderLeft - ((iconSeparation/2) * yRatio) + (xShiftRight/2) - (xShiftLeft/2), iconFloor - backBarBorder)
                karamba.moveImage(widget, backBarRight, borderLeft + (numIcons * iconSeparation)    + ((iconSeparation/2) * yRatio) - (xShiftRight/2) + (xShiftLeft/2), iconFloor - backBarBorder)
                karamba.resizeImage(widget, backBarCenter, widgetWidth - borderLeft - borderRight + (iconSeparation * yRatio) + xShiftLeft - xShiftRight + 1, imageSize + (backBarBorder * 2))
            
            
            ################################
            ## Size and position iconNum-1
            ################################
            
            ## Unless we're on the first image, resize the icon to our left
            if iconNum > 0:
                ## Calculate stuff
                ## Size is getting smaller
                size = imageSize + (yRatio * ((sizeDiff/2) - (offsetSizeDiff/2)))
                ## Icon is going left
                ## If we're positioning the last icon, do a special formula
                if (iconNum == numIcons):
                    xpos = ( ((iconNum-1-yRatio) * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) + (yRatio * (iconSeparation/2)) + (yRatio * ((iconSeparation/2) * offsetRatio)) )
                else:
                    ##        Initial placement                                    center it in it's cell          start half way accross and come back to initial placement
                    xpos = ( ((iconNum-1-yRatio) * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) + (yRatio * (iconSeparation/2)) - (yRatio * ((iconSeparation/2) * offsetRatio)) )
                ## Icon is going down
                ##     center icon                 start from half way up   and go half way down
                ypos = (iconFloor + imageSize - size) - (yRatio * (raiseIcons/2)) + ((yRatio * raiseIcons * offsetRatio)/2)
                
                
                ## Place stuff
                karamba.resizeImage(widget, imagesBig[iconNum-1], size, size)
                karamba.moveImage(widget, imagesBig[iconNum-1], xpos, ypos)
                
                ## Make sure there are no left-over image effects on the image
                if (conf["currentIconEffect"] != "none"):
                    karamba.removeImageEffects(widget, imagesBig[iconNum-1])
                
                ## Make sure the current small icon is off the screen
                karamba.moveImage(widget, images[iconNum-1], ((iconNum-1)*iconSeparation)+imageSizeBig, widgetHeight)
                
                ## Make sure all the previous icons are setup normally
                
                for i in range (0, iconNum - 1):
                    karamba.moveImage(widget, images[i], ((i-yRatio)*(iconSeparation)) + borderLeft + centerSmallIcon + xShiftRight, iconFloor)
                    karamba.moveImage(widget, imagesBig[i], ((i-1)*(iconSeparation)) + borderLeft, widgetHeight)
                
                if (conf['debug'] >= 2):
                    print "iconNum: ", iconNum-1, "\toffset: ", offset, "\toffsetSizeDiff: ", offsetSizeDiff, "\toffsetImageSizeBig: ", offsetImageSizeBig, "\tyRatio: ", yRatio, "\tsize: ", size, "\txpos: ", xpos, "\typos: ", ypos
            
            ################################
            ## Size and position iconNum
            ################################
            
            if ( (iconNum > -1) and (iconNum < numIcons) ):
                ## If we're on the left side of the icon
                if offset < (iconSeparation/2):
                    ## Calculate stuff
                    ## Size is getting larger
                    size = imageSize + (yRatio * ((sizeDiff/2) + offsetSizeDiff))
                    ## Icon is moving right
                    xpos = ( (iconNum * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) + (yRatio * (iconSeparation/2)) - (yRatio * offset) )
                    ## Icon is going up
                    ypos = (iconFloor + imageSize - size) - (yRatio * (raiseIcons/2)) - ((yRatio * raiseIcons * offsetRatio)/2)
                    
                    ## Place stuff
                    karamba.resizeImage(widget, imagesBig[iconNum], size, size)
                    karamba.moveImage(widget, imagesBig[iconNum], xpos, ypos)
                
                
                ## If we're on the right side of the icon
                else:
                    ## Calculate stuff
                    ## Size is getting smaller
                    size = imageSize + (yRatio * (sizeDiff + (sizeDiff/2) - offsetSizeDiff))
                    ##        initial placement                         center it                       go half way accross and come back offset pixels
                    xpos = ( (iconNum * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) + (yRatio * (iconSeparation/2)) - (yRatio * offset) )
                    ## icon is going down
                    ypos = (iconFloor + imageSize - size) - (yRatio * raiseIcons) + ((yRatio * raiseIcons * offsetRatio)/2)
                    
                    ## Place stuff
                    karamba.resizeImage(widget, imagesBig[iconNum], size, size)
                    karamba.moveImage(widget, imagesBig[iconNum], xpos, ypos)
                
                ## Apply current icon effects if any
                if (conf["currentIconEffect"] != "none"):
                    if (conf["currentIconEffect"] == "highlight"):
                        karamba.changeImageIntensity(widget, imagesBig[iconNum], 0.2, 0)
                    if (conf["currentIconEffect"] == "gray"):
                        karamba.changeImageToGray(widget, imagesBig[iconNum], 0)
                
                ## Move the current small image off the widget
                karamba.moveImage(widget, images[iconNum], (iconNum*iconSeparation) + borderLeft + centerSmallIcon, widgetHeight)
                
                ## These are used later if the icon is clicked
                currentXPosition = xpos
                currentYPosition = ypos
                currentSize = size
                
                if (conf['debug'] >= 2):
                    print "iconNum: ", iconNum, "\toffset: ", offset, "\toffsetSizeDiff: ", offsetSizeDiff, "\toffsetImageSizeBig: ", offsetImageSizeBig, "\tyRatio: ", yRatio, "\tsize: ", size, "\txpos: ", xpos, "\typos: ", ypos
                
            
            
            
            
            ################################
            ## Size and position iconNum+1
            ################################
            
            ## Unless we're on the last image, resize the icon to our right
            if (iconNum < numIcons-1):
                ## Calculate stuff
                size = imageSize + (yRatio * ((offsetSizeDiff/2)))
                ## If we're positioning the first icon, do a special formula
                if (iconNum == -1):
                    xpos = ( ((iconNum+1) * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) + (yRatio * (iconSeparation/2) * offsetRatio) )
                else:
                    ## icon is going left
                    xpos = ( ((iconNum+1+yRatio) * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) - (yRatio * (iconSeparation/2) * offsetRatio) )   ## Moves to the left as offset increases
                ## icon is going up
                ypos = (iconFloor + imageSize - size) - ((yRatio * raiseIcons * offsetRatio)/2)
                
                
                ## Place stuff
                karamba.resizeImage(widget, imagesBig[iconNum+1], size, size)
                karamba.moveImage(widget, imagesBig[iconNum+1], xpos, ypos)
                
                ## Make sure there are no left-over image effects on the image
                if (conf["currentIconEffect"] != "none"):
                    karamba.removeImageEffects(widget, imagesBig[iconNum+1])
                
                ## Make sure the current small icon is off the screen
                karamba.moveImage(widget, images[iconNum+1], ((iconNum+1)*iconSeparation)+imageSizeBig, widgetHeight)
                
                ## Make sure all the following icons are setup normally
                
                for i in range (iconNum+2, numIcons):
                    karamba.moveImage(widget, images[i], ((i-1+yRatio)*iconSeparation) + iconSeparation + borderLeft + centerSmallIcon + xShiftLeft, iconFloor)
                    karamba.moveImage(widget, imagesBig[i], ((i)*iconSeparation) + iconSeparation + borderLeft, widgetHeight)
                    
                if (conf['debug'] >= 2):
                    print "iconNum: ", iconNum+1, "\toffset: ", offset, "\toffsetSizeDiff: ", offsetSizeDiff, "\toffsetImageSizeBig: ", offsetImageSizeBig, "\tyRatio: ", yRatio, "\tsize: ", size, "\txpos: ", xpos, "\typos: ", ypos
            if (conf['debug'] >= 2):
                print
            
            
            ## Update the screen with the changes we've just made
            karamba.redrawWidget(widget)
            
# This was a test on highlighting the desktop icon the mouse was currently over    
#    
#     ################################
#     ##   Update Virtual Desktops  ##
#     ################################
#     elif ( (conf["virtualDesktopSelector"] == 1) and (x >= xVdBoundaryMin) and (x < xVdBoundaryMax) and (y >= yVdBoundaryMin) and (y <= yVdBoundaryMax) ):
#         
#         ## Determine which icon we're over and highlight it
#         icon = int((x - xVdBoundaryMin) / conf["virtualDesktopIconX"])
#         if (y > yVdBoundaryMax - conf["virtualDesktopIconY"]):
#             icon += (conf["numVirtualDesktops"]/2)
#         icon += 1
#         
#         ## Update the screen with the changes we've just made
#         karamba.redrawWidget(widget)
        
            
    
    ## If mouse is to far right or left reset everything to normal
    else:
        if (conf['debug'] >= 1):
            print "cursor is out of bounds. x: ", x, " y: ", y
        resetWidget(widget)
        
    
    ## Sleep a bit to accomadate the maxFPS setting
    if (sleepTime):
        time.sleep(sleepTime)
    










#############################################################################
##  Function:
##    resetWidget(widget)
##  
##  Description:
##    Resets the widget to it's normal state, i.e. no icons zoomed
##  
#############################################################################
def resetWidget(widget):
    """Resets widget to standby state.. i.e. no icons zoomed."""    
    
    for i in range (0,numIcons):
        karamba.moveImage(widget, images[i], (i*iconSeparation) + borderLeft + centerSmallIcon, iconFloor)
        karamba.moveImage(widget, imagesBig[i], (i*iconSeparation) + borderLeft, widgetHeight)
    
    if (useBackBar == 1):
        karamba.moveImage(widget, backBarLeft, borderLeft - imageSize - (backBarBorder * 2), iconFloor - backBarBorder)
        karamba.moveImage(widget, backBarCenter, borderLeft, iconFloor - backBarBorder)
        karamba.moveImage(widget, backBarRight, borderLeft + (numIcons * iconSeparation), iconFloor - backBarBorder)
        karamba.resizeImage(widget, backBarCenter, widgetWidth - borderLeft - borderRight, imageSize + (backBarBorder * 2))
    
    karamba.redrawWidget(widget)










###############################################################################
## Function:    
##   readConfigurationFile ( string $filename )
## 
## 
## Description: 
##   Reads the file $filename, and stores information from the file into 
##   the global hash %conf.  This version of the function has been modified 
##   specifically to accomadate the needs of Kroller.
## 
## Example: 
##   readConfigurationFile("/etc/configuration.conf");
##
###############################################################################
def readConfigurationFile(filename):
    """Read a configuration file and store name:value pairs into the global variable conf"""
    global conf
    global commands
    global imageFiles
    global imageFilesBig
    
    ## Open the configuration file
    try:
        file = open(filename,"r")
    except IOError:
        print
        print "ERROR While opening the configuration file [" + filename + "]"
        print "Hint: try putting kroller.conf in ~/.superkaramba/"
        print
    
    ## Read through each line of the file
    while 1:
        line = file.readline()                        ## Read a line from the file
        if len(line) == 0:                            ## Break this while loop if we're at the end of the file
            break
        
        line = re.sub("#.*$", "", line)               ## Remove comments
        line = re.sub("//.*$", "", line)              ## Remove comments
        line = line.strip()                           ## Remove line ending characters, and extra whitespace
        if re.search("\s+=>\s+", line):               ## If it looks like a valid configuration line
            
            ## Debugging
            if (conf['debug'] >= 6):
                print "Line is: ", line
            
            ## Extract the name/value pair
            (name, value) = re.split("\s+=>\s+", line, 1)
            
            ## Add a command
            if (name == "addCommand"):
                commands.append(value)
            
            ## Add a small image
            elif (name == "addSmallImage"):
                imageFiles.append(value)
            
            ## Add a large image
            elif (name == "addLargeImage"):
                imageFilesBig.append(value)
            
            ## Must be a generic configuration parameter, store it in conf
            else:
                if (re.match("^\d+$", value)):
                    conf[name] = int(value)
                else:
                    conf[name] = value
        
    ## Close the coniguration file
    file.close()
    
    
    ## DEBUGGING:
    if conf['debug'] >= 3:
        print "Configuration: "
        for key in conf.keys():
            print key, "\t=>\t", conf[key]










###############################################################################
## Function:    
##   detectResolution ( )
## 
## 
## Description: 
##   Runs xwininfo and sets the current XFree86 resolution to the golbal
##   variables resolutionX and resolutionY
## 
##  Thanks to Adam Geitgey for this code!
###############################################################################
def detectResolution():
    """Detect Current XFree86 Resolution"""
    global resolutionX
    global resolutionY
    
    havexwininfo = os.system("which xwininfo >/dev/null")
    
    if (havexwininfo == 0):
        pass
    else:
        resolutionX = 1024
        resolutionY = 768
        print "\nCan't find xwininfo in your path!  Guessing resolution is 1024x768."
    
    fp = os.popen("xwininfo -root -stats")
    output = fp.read()
    output = output.splitlines()
    i = 0
    for x in output:
        param = x.split()
        if (len(param) > 1):
            if param[0].find("Width:") != -1:
                resolutionX = int(param[1])
            if param[0].find("Height:") != -1:
                resolutionY = int(param[1])
    
    print "Dected resolution: " + str(resolutionX) + "x" + str(resolutionY)









###############################################################################
## Function:    
##   hideWidget ( widget )
## 
## 
## Description: 
##   Hides kroller
## 
###############################################################################
def hideWidget(widget):
    
    karamba.moveWidget(widget, ((resolutionX/2) - (widgetWidth/2)), (resolutionY-borderBottom) )
    
    
    







###############################################################################
## Function:    
##   raiseWidget ( widget )
## 
## 
## Description: 
##   Shows kroller
## 
###############################################################################
def raiseWidget(widget):
    
    karamba.moveWidget(widget, ((resolutionX/2) - (widgetWidth/2)), (resolutionY-widgetHeight) )










# This will be printed when the widget loads.
print "Kroller python module loaded!"

